@@ -6,18 +6,19 @@ from pathlib import Path
 from typing import Any, List, Optional, Tuple
 
 from ._logger import logger
-from .constants import (
-    AIOPYTESSERACT_DEFAULT_ENCODING,
-    AIOPYTESSERACT_DEFAULT_TIMEOUT,
-    OUTPUT_FILE_EXTENSIONS,
-    TESSERACT_CMD,
-)
-from .exceptions import TesseractNotFoundError, TesseractRuntimeError
-from .validators import file_exists, language_is_valid, oem_is_valid, psm_is_valid
+from .constants import (AIOPYTESSERACT_DEFAULT_ENCODING,
+                        AIOPYTESSERACT_DEFAULT_TIMEOUT, OUTPUT_FILE_EXTENSIONS,
+                        TESSERACT_CMD)
+from .exceptions import TesseractRuntimeError
+from .returncode import ReturnCode
+from .validators import (file_exists, language_is_valid, oem_is_valid,
+                         psm_is_valid)
 
 
 async def execute_cmd(cmd_args: str, timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT):
-    logger.debug(f"Command: '{TESSERACT_CMD} {shlex.join(shlex.split(cmd_args))}'")
+    logger.debug(
+        f"aiopytesseract command: '{TESSERACT_CMD} {shlex.join(shlex.split(cmd_args))}'"
+    )
     proc = await asyncio.wait_for(
         asyncio.create_subprocess_exec(
             TESSERACT_CMD,
@@ -42,6 +43,7 @@ async def execute(
     timeout: float,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
 ) -> bytes:
     raise NotImplementedError
 
@@ -57,6 +59,7 @@ async def _(
     timeout: float,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
 ) -> bytes:
     await file_exists(image)
     response: bytes = await execute(
@@ -69,6 +72,7 @@ async def _(
         timeout,
         user_words,
         user_patterns,
+        tessdata_dir,
     )
     return response
 
@@ -84,6 +88,8 @@ async def _(
     timeout: float,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> bytes:
     cmd_args = await _build_cmd_args(
         output_extension=output_format,
@@ -92,24 +98,22 @@ async def _(
         oem=oem,
         user_words=user_words,
         user_patterns=user_patterns,
+        tessdata_dir=tessdata_dir,
         lang=lang,
     )
-    try:
-        proc = await asyncio.wait_for(
-            asyncio.create_subprocess_exec(
-                TESSERACT_CMD,
-                *cmd_args,
-                stdin=asyncio.subprocess.PIPE,
-                stdout=asyncio.subprocess.PIPE,
-                stderr=asyncio.subprocess.PIPE,
-            ),
-            timeout=timeout,
-        )
-    except OSError:
-        raise TesseractNotFoundError(f"{TESSERACT_CMD} not found.")
+    proc = await asyncio.wait_for(
+        asyncio.create_subprocess_exec(
+            TESSERACT_CMD,
+            *cmd_args,
+            stdin=asyncio.subprocess.PIPE,
+            stdout=asyncio.subprocess.PIPE,
+            stderr=asyncio.subprocess.PIPE,
+        ),
+        timeout=timeout,
+    )
     stdout, stderr = await asyncio.wait_for(proc.communicate(image), timeout=timeout)
-    if proc.returncode != 0:
-        raise TesseractRuntimeError(stderr.decode(AIOPYTESSERACT_DEFAULT_ENCODING))
+    if proc.returncode != ReturnCode.SUCCESS:
+        raise TesseractRuntimeError(stderr.decode(encoding))
     return stdout
 
 
@@ -124,6 +128,8 @@ async def execute_multi_output_cmd(
     timeout: float,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> Tuple[str, ...]:
     cmd_args = await _build_cmd_args(
         output_extension=output_format,
@@ -132,25 +138,23 @@ async def execute_multi_output_cmd(
         oem=oem,
         user_words=user_words,
         user_patterns=user_patterns,
+        tessdata_dir=tessdata_dir,
         lang=lang,
         output=output_file,
     )
-    try:
-        proc = await asyncio.wait_for(
-            asyncio.create_subprocess_exec(
-                TESSERACT_CMD,
-                *cmd_args,
-                stdin=asyncio.subprocess.PIPE,
-                stdout=asyncio.subprocess.PIPE,
-                stderr=asyncio.subprocess.PIPE,
-            ),
-            timeout=timeout,
-        )
-    except OSError:
-        raise TesseractNotFoundError(f"{TESSERACT_CMD} not found.")
+    proc = await asyncio.wait_for(
+        asyncio.create_subprocess_exec(
+            TESSERACT_CMD,
+            *cmd_args,
+            stdin=asyncio.subprocess.PIPE,
+            stdout=asyncio.subprocess.PIPE,
+            stderr=asyncio.subprocess.PIPE,
+        ),
+        timeout=timeout,
+    )
     _, stderr = await asyncio.wait_for(proc.communicate(image), timeout=timeout)
-    if proc.returncode != 0:
-        raise TesseractRuntimeError(stderr.decode(AIOPYTESSERACT_DEFAULT_ENCODING))
+    if proc.returncode != ReturnCode.SUCCESS:
+        raise TesseractRuntimeError(stderr.decode(encoding))
     return tuple(
         [f"{output_file}{OUTPUT_FILE_EXTENSIONS[ext]}" for ext in output_format.split()]  # type: ignore
     )
@@ -163,6 +167,7 @@ async def _build_cmd_args(
     oem: int,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
     lang: Optional[str] = None,
     output: str = "stdout",
 ) -> List[str]:
@@ -178,6 +183,10 @@ async def _build_cmd_args(
         cmd_args.append("--user-patterns")
         cmd_args.append(user_patterns)
 
+    if tessdata_dir:
+        cmd_args.append("--tessdata-dir")
+        cmd_args.append(tessdata_dir)
+
     if lang:
         await language_is_valid(lang)
         cmd_args.append("-l")
@@ -187,5 +196,5 @@ async def _build_cmd_args(
     for ext in extension:
         cmd_args.append(ext)
 
-    logger.debug(f"Command: 'tesseract {shlex.join(cmd_args)}'")
+    logger.debug(f"aiopytesseract command: 'tesseract {shlex.join(cmd_args)}'")
     return shlex.split(shlex.join(cmd_args))
