@@ -8,6 +8,8 @@ from typing import Any, AsyncGenerator, List, Optional, Tuple
 import cattr
 from aiofiles import tempfile  # type: ignore
 
+from aiopytesseract.validators import file_exists
+
 from .base_command import execute, execute_cmd, execute_multi_output_cmd
 from .constants import (
     AIOPYTESSERACT_DEFAULT_DPI,
@@ -21,36 +23,41 @@ from .constants import (
 from .exceptions import TesseractRuntimeError
 from .file_format import FileFormat
 from .models import OSD, Box, Data, Parameter
+from .returncode import ReturnCode
 
 
-async def languages(config: str = "") -> List:
+async def languages(
+    config: str = "", encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING
+) -> List:
     """Tesseract available languages."""
     proc = await execute_cmd(f"--list-langs {config}")
     data = await proc.stdout.read()
     languages = []
-    for line in data.decode(AIOPYTESSERACT_DEFAULT_ENCODING).split():
+    for line in data.decode(encoding).split():
         lang = line.strip()
         if lang in TESSERACT_LANGUAGES:
             languages.append(lang)
     return languages
 
 
-async def get_languages(config: str = "") -> List:
+async def get_languages(
+    config: str = "", encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING
+) -> List:
     """Tesseract available languages."""
-    langs = await languages(config)
+    langs = await languages(config, encoding=encoding)
     return langs
 
 
-async def tesseract_version() -> str:
+async def tesseract_version(encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING) -> str:
     """Tesseract version."""
     proc = await execute_cmd("--version")
     data: bytes = await proc.stdout.readuntil()
-    return data.decode(AIOPYTESSERACT_DEFAULT_ENCODING).split()[1]
+    return data.decode(encoding).split()[1]
 
 
-async def get_tesseract_version() -> str:
+async def get_tesseract_version(encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING) -> str:
     """Tesseract version."""
-    version = await tesseract_version()
+    version = await tesseract_version(encoding)
     return version
 
 
@@ -60,6 +67,7 @@ async def confidence(
     lang: str = AIOPYTESSERACT_DEFAULT_LANGUAGE,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> float:
     """Get script confidence.
 
@@ -69,14 +77,14 @@ async def confidence(
     :param timeout: command timeout (default: 30)
     """
     proc = await execute_cmd(f"stdin stdout -l {lang} --dpi {dpi} --psm 0 --oem {oem}")
-    stdout, stderr = await asyncio.wait_for(
+    stdout, _ = await asyncio.wait_for(
         proc.communicate(Path(image).read_bytes()), timeout=timeout
     )
     try:
         confidence_value = float(
             re.search(  # type: ignore
                 r"(Script.confidence:.(\d{1,10}.\d{1,10})$)",
-                stdout.decode(AIOPYTESSERACT_DEFAULT_ENCODING),
+                stdout.decode(encoding),
             ).group(2)
         )
     except AttributeError:
@@ -90,6 +98,7 @@ async def deskew(
     lang: str = AIOPYTESSERACT_DEFAULT_LANGUAGE,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> float:
     """Get Deskew angle.
 
@@ -107,7 +116,7 @@ async def deskew(
         deskew_value = float(
             re.search(  # type: ignore
                 r"(Deskew.angle:.)(\d{1,10}.\d{1,10}$)",
-                data.decode(AIOPYTESSERACT_DEFAULT_ENCODING),
+                data.decode(encoding),
             ).group(2)
         )
     except AttributeError:
@@ -115,14 +124,16 @@ async def deskew(
     return deskew_value
 
 
-async def tesseract_parameters() -> List[Parameter]:
+async def tesseract_parameters(
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
+) -> List[Parameter]:
     """List of all Tesseract parameters with default value and short description.
 
     reference: https://tesseract-ocr.github.io/tessdoc/tess3/ControlParams.html
     """
     proc = await execute_cmd("--print-parameters")
     raw_data: bytes = await proc.stdout.read()
-    data = raw_data.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
+    data = raw_data.decode(encoding)
     params = []
     for line in data.split("\n"):
         param = re.search(r"(\w+)\s+(-?\d+.?\d{0,})\s+(.*)[^\n]$", line)
@@ -140,6 +151,7 @@ async def image_to_string(
     image: Any,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     lang: str = AIOPYTESSERACT_DEFAULT_LANGUAGE,
     psm: int = AIOPYTESSERACT_DEFAULT_PSM,
@@ -170,6 +182,8 @@ async def _(
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> str:
     image_text: bytes = await execute(
         image,
@@ -181,8 +195,9 @@ async def _(
         timeout,
         user_words,
         user_patterns,
+        tessdata_dir,
     )
-    return image_text.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
+    return image_text.decode(encoding)
 
 
 @image_to_string.register(bytes)
@@ -195,6 +210,8 @@ async def _(
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> str:
     image_text: bytes = await execute(
         image,
@@ -206,8 +223,9 @@ async def _(
         timeout,
         user_words,
         user_patterns,
+        tessdata_dir,
     )
-    return image_text.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
+    return image_text.decode(encoding)
 
 
 @singledispatch
@@ -215,6 +233,7 @@ async def image_to_hocr(
     image: Any,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     lang: str = AIOPYTESSERACT_DEFAULT_LANGUAGE,
     psm: int = AIOPYTESSERACT_DEFAULT_PSM,
@@ -240,11 +259,13 @@ async def _(
     image: str,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     lang: str = AIOPYTESSERACT_DEFAULT_LANGUAGE,
     psm: int = AIOPYTESSERACT_DEFAULT_PSM,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> str:
     output: bytes = await execute(
         image,
@@ -256,8 +277,9 @@ async def _(
         timeout,
         user_words,
         user_patterns,
+        tessdata_dir,
     )
-    return output.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
+    return output.decode(encoding)
 
 
 @image_to_hocr.register(bytes)
@@ -270,6 +292,8 @@ async def _(
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> str:
     output: bytes = await execute(
         image,
@@ -281,8 +305,9 @@ async def _(
         timeout,
         user_words,
         user_patterns,
+        tessdata_dir,
     )
-    return output.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
+    return output.decode(encoding)
 
 
 @singledispatch
@@ -295,6 +320,7 @@ async def image_to_pdf(
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
 ) -> bytes:
     """Generate a searchable PDF from an image.
 
@@ -320,6 +346,7 @@ async def _(
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
 ) -> bytes:
     output: bytes = await execute(
         image,
@@ -331,6 +358,7 @@ async def _(
         timeout,
         user_words,
         user_patterns,
+        tessdata_dir,
     )
     return output
 
@@ -345,6 +373,7 @@ async def _(
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
 ) -> bytes:
     output: bytes = await execute(
         image,
@@ -356,6 +385,7 @@ async def _(
         timeout,
         user_words,
         user_patterns,
+        tessdata_dir,
     )
     return output
 
@@ -374,17 +404,22 @@ async def image_to_boxes(
 
 @image_to_boxes.register(str)
 async def _(image: str, timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT) -> List[Box]:
+    await file_exists(image)
     boxes = await image_to_boxes(Path(image).read_bytes(), timeout)
     return boxes
 
 
 @image_to_boxes.register(bytes)
-async def _(image: bytes, timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT) -> List[Box]:
+async def _(
+    image: bytes,
+    timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
+) -> List[Box]:
     proc = await execute_cmd("stdin stdout batch.nochop makebox")
     stdout, stderr = await asyncio.wait_for(proc.communicate(image), timeout=timeout)
-    if proc.returncode != 0:
-        raise TesseractRuntimeError(stderr.decode(AIOPYTESSERACT_DEFAULT_ENCODING))
-    data = stdout.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
+    if proc.returncode != ReturnCode.SUCCESS:
+        raise TesseractRuntimeError(stderr.decode(encoding))
+    data = stdout.decode(encoding)
     datalen = len(data.split("\n")) - 1
     boxes = []
     for line in data.split("\n")[:datalen]:
@@ -412,6 +447,7 @@ async def _(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
 ) -> List[Data]:
+    await file_exists(image)
     data_values = await image_to_data(Path(image).read_bytes(), dpi, timeout)
     return data_values
 
@@ -421,12 +457,13 @@ async def _(
     image: bytes,
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> List[Data]:
     proc = await execute_cmd(f"stdin stdout -c tessedit_create_tsv=1 --dpi {dpi}")
     stdout, stderr = await asyncio.wait_for(proc.communicate(image), timeout=timeout)
-    if proc.returncode != 0:
-        raise TesseractRuntimeError(stderr.decode(AIOPYTESSERACT_DEFAULT_ENCODING))
-    data: str = stdout.decode(AIOPYTESSERACT_DEFAULT_ENCODING)
+    if proc.returncode != ReturnCode.SUCCESS:
+        raise TesseractRuntimeError(stderr.decode(encoding))
+    data: str = stdout.decode(encoding)
     datalen = len(data.split("\n")) - 1
     params = []
     for line in data.split("\n")[1:datalen]:
@@ -441,6 +478,7 @@ async def image_to_osd(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> OSD:
     """Information about orientation and script detection.
 
@@ -458,15 +496,10 @@ async def _(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> OSD:
-    data = await execute(image, FileFormat.OSD, dpi, None, 0, oem, timeout)
-    osd = cattr.structure_attrs_fromtuple(
-        re.findall(  # type: ignore
-            r"\w+\s?\:\s{0,}(\d+.?\d{0,}|\w+)",
-            data.decode(AIOPYTESSERACT_DEFAULT_ENCODING),
-        ),
-        OSD,
-    )
+    await file_exists(image)
+    osd = await image_to_osd(Path(image).read_bytes(), dpi, oem, timeout, encoding)
     return osd
 
 
@@ -476,12 +509,13 @@ async def _(
     dpi: int = AIOPYTESSERACT_DEFAULT_DPI,
     oem: int = AIOPYTESSERACT_DEFAULT_OEM,
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
+    encoding: str = AIOPYTESSERACT_DEFAULT_ENCODING,
 ) -> OSD:
     data = await execute(image, FileFormat.OSD, dpi, None, 0, oem, timeout)
     osd = cattr.structure_attrs_fromtuple(
         re.findall(  # type: ignore
             r"\w+\s?\:\s{0,}(\d+.?\d{0,}|\w+)",
-            data.decode(AIOPYTESSERACT_DEFAULT_ENCODING),
+            data.decode(encoding),
         ),
         OSD,
     )
@@ -500,6 +534,7 @@ async def run(
     timeout: float = AIOPYTESSERACT_DEFAULT_TIMEOUT,
     user_words: Optional[str] = None,
     user_patterns: Optional[str] = None,
+    tessdata_dir: Optional[str] = None,
 ) -> AsyncGenerator[Tuple[str, ...], None]:
     """Run Tesseract-OCR with multiple analysis.
 
@@ -531,5 +566,6 @@ async def run(
             timeout,
             user_words,
             user_patterns,
+            tessdata_dir,
         )
         yield resp
