@@ -15,7 +15,12 @@ async def test_execute_unsupported(input_data):
 @pytest.mark.asyncio
 async def test_build_cmd_args_with_user_patterns():
     command = await aiopytesseract.base_command._build_cmd_args(
-        "stdout", 200, 3, 3, user_patterns="tests/samples/user_patterns.txt"
+        "stdout",
+        200,
+        3,
+        3,
+        user_patterns="tests/samples/user_patterns.txt",
+        tessdata_dir="/tesser-test-data",
     )
     assert command == [
         "stdin",
@@ -28,5 +33,7 @@ async def test_build_cmd_args_with_user_patterns():
         "3",
         "--user-patterns",
         "tests/samples/user_patterns.txt",
+        "--tessdata-dir",
+        "/tesser-test-data",
         "stdout",
     ]
